%% -------------------------
%% function MOTOHAWK_AUTODOC
%% -------------------------
%%
function motohawk_autodoc(ModelName, p1, v1, p2, v2)
% --- Takes screen shots of model

% --------------
% --- Load Model
% --------------

try
    ModelName = ModelName;
catch
    ModelName = bdroot;
end
try
    open_system(ModelName);
    disp(['AutoDoc process started for: ' ModelName]);
catch
    disp('Error: The specified model is not open and/or cannot be found on the current path');
    return
end
islibrary = strcmp(get_param(ModelName, 'LibraryType'), 'BlockLibrary');
if islibrary && strcmp(get_param(ModelName, 'Lock'), 'on')
    disp('Error: Model is a locked library; please unlock before proceeding.');
    return
end
load_system('AutoDoc_lib');

% ----------------------
% --- Create directories
% ----------------------

[fldr, nam, ext, ver] = fileparts(which(ModelName));
pth = [fldr '\AutoDoc_' ModelName];
if ~ isdir(pth)
    mkdir(pth);
end
copyfile(which('AutoDoc_SummaryTemplate.xls'), [pth '\AutoDoc_Summary.xls']);

% ----------------------------------------------
% --- Automatically Fill Model with Title Blocks
% ----------------------------------------------

try
    autofill = (strcmp(p1, 'AutoFill') && strcmp(v1, 'on')) || (strcmp(p2, 'AutoFill') && strcmp(v2, 'on'));
catch
    autofill = 0;
end
if autofill
    blklst = [ModelName; find_system(ModelName, 'LookUnderMasks', 'all', 'BlockType', 'SubSystem', 'ReferenceBlock', '')];
    for i = 1:length(blklst)
        if isempty(find_system(blklst{i}, 'SearchDepth', 1, 'LookUnderMasks', 'all', 'ReferenceBlock', 'AutoDoc_lib/motohawk_autodoc'))
            add_block('AutoDoc_lib/motohawk_autodoc', [blklst{i} '/motohawk_autodoc']);
        end
    end
end

% --------------------------------------
% --- Find and sort documentation blocks
% --------------------------------------

autodocblklst = find_system(ModelName, 'LookUnderMasks', 'all', 'ReferenceBlock', 'AutoDoc_lib/motohawk_autodoc');
count = 0;
autodocblklst_indoc = {};
mdldpth = 0;
for i = 1:length(autodocblklst)
    UserData = get_param(autodocblklst{i}, 'UserData');
    if UserData.DocOption == 1
        count = count + 1;
        autodocblklst_indoc{count} = autodocblklst{i};
        mdldpth = max([mdldpth; length(strfind(UserData.DocPath, '.'))]);
    end
    clear UserData
end
mdldpth = mdldpth + 1;
autodocblklst = autodocblklst_indoc;

% -----------------
% --- Print screens
% -----------------

disp('Taking screen shots...');

docpath = {};
for i = 1:length(autodocblklst)
    
    % update AutoDoc blocks
    open_system(autodocblklst{i});
    scrnpth = get_param(autodocblklst{i}, 'Parent');
    UserData = get_param(autodocblklst{i}, 'UserData');
    docpath{i} = UserData.DocPath;
    emfnam = strrep(docpath{i}, '.00', '.01');        
    emfnamdpth = length(strfind(emfnam, '.'));
    while emfnamdpth < mdldpth
        emfnam = [emfnam '.00'];
        emfnamdpth = length(strfind(emfnam, '.'));
    end    
    clear UserData    
    
    % retrieve docpathdisp from title block for table of contents
    titleblkpth = find_system(scrnpth, 'SearchDepth', 1, 'LookUnderMasks', 'all', 'ReferenceBlock', 'AutoDoc_lib/autodoc_titleblk');
    UserData = get_param(titleblkpth{1}, 'UserData');
    docpathdisp{i} = UserData.DocPathDisp;
    docpathdisp{i} = strrep(docpathdisp{i}, '\_', '_');
    clear UserData;
    
    % open and print
    open_system(scrnpth, 'force');
    h = get_param(scrnpth, 'Handle');
    try
        print(h, '-v', '-dmeta', [pth, '\', emfnam, '.emf']);
    catch
    end
    docpathpages(i) = 1;
    
    % find cals and probes page
    cpblkpth = find_system(scrnpth, 'SearchDepth', 1, 'LookUnderMasks', 'all', 'ReferenceBlock', 'AutoDoc_lib/autodoc_calsprbspage');
    
    % open, print, and close cals and probes page
    if ~ isempty(cpblkpth)
        open_system(cpblkpth{1}, 'force');
        h = get_param(cpblkpth{1}, 'Handle');
        print(h, '-v', '-dmeta', [pth, '\', emfnam(1:(end-2)), 'cp.emf']);
        close_system(cpblkpth{1});
        docpathpages(i) = docpathpages(i) + 1;
    end
    
    % find Stateflow diagrams
    sfblkpths = find_system(scrnpth, 'SearchDepth', 1, 'LookUnderMasks', 'all', 'MaskType', 'Stateflow', 'MaskDescription', 'Stateflow diagram');
    
    % open, print, and close Stateflow diagrams
    currdir = cd;
    cd(pth);
    for j = 1:length(sfblkpths)
        sfpicfilenam = [emfnam(1:(end-2)), 's', num2str(j), '.emf'];
        open_system(sfblkpths{j});
        sfprint(sfblkpths{j}, 'meta', sfpicfilenam);
        sfclose;
        docpathpages(i) = docpathpages(i) + 1;
    end
    cd(currdir);
    
    % close
    if ~ strcmp(ModelName, scrnpth)
        close_system(scrnpth)
    end
    
end

% ----------------------------
% --- Create Table of Contents
% ----------------------------

disp('Creating table of contents document...');

% display strings
dotnum = 4 * fliplr([26:46]);
dots = {};
for i = 1:length(dotnum)
    dotdisp = '';
    for j = 1:dotnum(i)
        dotdisp = [dotdisp '.'];
    end
    dots{i} = dotdisp;
end

% sort by docpath
[docpath, idx] = sort(docpath);

% starting page
try
    if strcmp(p1, 'StartPg')
        page = v1;
    elseif strcmp(p2, 'StartPg')
        page = v2;
    end
catch
    page = 1;
end

% print lines
fid = fopen([pth, '\AutoDoc_TableOfContents.doc'], 'w+');
for i = 1:length(docpath)
    if strcmp(docpath{i}, '00')
        level = 0;
    elseif strncmp(docpath{i}, '00', 2)
        level = 1;
    else
        level = length(strfind(docpath{i}, '.')) + 1;
    end
    indent = '';
    for j = 1:level
        indent = [indent '    '];
    end
    fprintf(fid, '%s \n', [indent docpathdisp{idx(i)}]);
    fprintf(fid, '%s %3.0f\n', [indent dots{level + 1}], page);
    page = page + docpathpages(idx(i));
end
    
fclose(fid);

% ----------------------------------------------
% --- Create Calibration, Probe, & Fault Summary
% ----------------------------------------------

fn = [pth, '\AutoDoc_Summary'];

% display format variables
lngthcalnam = 50;
lngthprbnam = 50;
lngthfltnam = 64;
lngthunits = 19;
lngthhelp = 116;

% clear current data
xlswrite(fn, cellstr(''), 1, 'B7:H5001');
xlswrite(fn, cellstr(''), 2, 'B7:H5001');
xlswrite(fn, cellstr(''), 3, 'B4:E499');
xlswrite(fn, cellstr(''), 3, 'G4:H99');

if ~ islibrary
    
    disp('Creating calibrations, probes, & faults summary spreadsheet...');

    % gather data
    [caldata prbdata] = calprbdata(ModelName, 50, 'all', 'on');
    fltlst = find_system(ModelName, 'LookUnderMasks', 'all', 'FollowLinks', 'on', 'MaskType', 'MotoHawk Fault Definition');
    fltlst = [fltlst; find_system(ModelName, 'LookUnderMasks', 'all', 'FollowLinks', 'on', 'MaskType', 'MotoHawk OBD Fault Definition')];
    fltactnlst = find_system(ModelName, 'LookUnderMasks', 'all', 'FollowLinks', 'on', 'MaskType', 'MotoHawk Fault Action');
    fltactnlst = [fltactnlst; find_system(ModelName, 'LookUnderMasks', 'all', 'FollowLinks', 'on', 'MaskType', 'MotoHawk OBD Fault Action Definition')];
    
    for i = 1:2
        clear data
        if i == 1
            lst = fltlst;
        else
            lst = fltactnlst;
        end
        data = [];
        for j = 1:length(lst)
            data.nam{j} = motohawk_get_param_ws(lst{j}, 'nam');
            if isempty(data.nam{j})
                data.nam{j} = 'Could Not Eval. (Update Reqd.)';
            end
        end
        if ~isempty(data)
            [data.nam idx] = sort(data.nam);
        end
        for j = 1:length(lst)
            data.mdlpath{j} = oneline(get_param(lst{idx(j)}, 'Parent'));
        end
        if i == 1
            fltdata = data;
        else
            fltactndata = data;
        end
    end
               
    % write calibration data
    txt = {};
    data = caldata;
    count = 0;
    if ~ isempty(data)
        for j = 1:length(data.nam)
            count = count + 1;
            txt{count,1} = num2str(j);
            txt{count,2} = textend(data.nam{j}, lngthcalnam, 0);
            txt{count,5} = textend(data.units{j}, lngthunits, 0);
            txt{count,6} = data.size{j};
            txt{count,7} = data.datatyp{j};
            count = count + 1;
            txt{count,2} = data.prec{j};
            txt{count,3} = textend(data.gain{j}, lngthunits, 0);
            txt{count,4} = textend(data.offset{j}, lngthunits, 0);
            txt{count,5} = textend(data.exp{j}, lngthunits, 0);
            txt{count,6} = textend(data.min{j}, lngthunits, 0);
            txt{count,7} = textend(data.max{j}, lngthunits, 0);
            count = count + 1;
            txt{count,2} = textend(data.help{j}, lngthhelp, 0);
            count = count + 1;
            txt{count,2} = docpthformat(data.mdlpath{j}, ModelName, lngthhelp);
            count = count + 1;
            txt{count,2} = textend(data.group{j}, lngthhelp, 0);
        end
    xlswrite(fn, txt, 1, ['B7:H', num2str(count + 6)]);    
    end    
    
    % write display data
    txt = {};
    data = prbdata;
    count = 0;
    if ~ isempty(data)
        for j = 1:length(data.nam)
            count = count + 1;
            txt{count,1} = num2str(j);
            txt{count,2} = textend(data.nam{j}, lngthprbnam, 0);
            txt{count,5} = textend(data.units{j}, lngthunits, 0);
            txt{count,6} = data.size{j};
            txt{count,7} = data.datatyp{j};
            count = count + 1;
            txt{count,2} = data.prec{j};
            txt{count,3} = textend(data.gain{j}, lngthunits, 0);
            txt{count,4} = textend(data.offset{j}, lngthunits, 0);
            txt{count,5} = textend(data.exp{j}, lngthunits, 0);
            txt{count,6} = textend(data.stortyp{j}, lngthunits, 0);
            count = count + 1;
            txt{count,2} = textend(data.help{j}, lngthhelp, 0);
            count = count + 1;
            txt{count,2} = docpthformat(data.mdlpath{j}, ModelName, lngthhelp);
            count = count + 1;
            txt{count,2} = textend(data.group{j}, lngthhelp, 0);
        end
    xlswrite(fn, txt, 2, ['B7:H', num2str(count + 6)]);    
    end
    
    % write fault data
    txt = {};
    data = fltdata;
    count = 0;
    if ~ isempty(data)
        for j = 1:length(data.nam)
            count = count + 1;
            txt{count,1} = num2str(j);
            txt{count,2} = textend(data.nam{j}, lngthfltnam, 0);
            count = count + 1;
            txt{count,2} = docpthformat(data.mdlpath{j}, ModelName, lngthhelp);
        end
    xlswrite(fn, txt, 3, ['B4:C', num2str(count + 3)]);    
    end
    
    % write fault action data
    txt = {};
    data = fltactndata;
    count = 0;
    if ~ isempty(data)
        for j = 1:length(data.nam)
            count = count + 1;
            txt{count,1} = num2str(j);
            txt{count,2} = textend(data.nam{j}, lngthfltnam, 0);
            count = count + 1;
            txt{count,2} = docpthformat(data.mdlpath{j}, ModelName, lngthhelp);
        end
    xlswrite(fn, txt, 3, ['E4:F', num2str(count + 3)]);    
    end    

end

% % ----------------------
% % --- Create CAN Summary
% % ----------------------
%  Create a CAN summary in Excel, similar to calibration, probe, and fault summaries

disp(['     (files saved in ' pth ')']);
disp('AutoDoc is complete.');

end


%% ---------------
%% function TXTEND
%% ---------------
%%
function outtxt = textend(intxt, maxlength, dir)
% --- Limits length of text
if length(intxt) > maxlength
    % limit at end of string
    if dir == 0
        outtxt = [intxt(1:(maxlength - 3)) '...'];
    % limit at beginning of string
    else
        outtxt = ['...' intxt((end - maxlength + 4):end)];
    end
else
    outtxt = intxt;
end

end


%% ----------------
%% function ONELINE
%% ----------------
%%
function outtxt = oneline(intxt)
% --- Places text on a single line
nonchar = find(~isstrprop(intxt, 'graphic'));
blank = strfind(intxt, ' ');
for i = 1:length(nonchar)
    if isempty(find(blank == nonchar(i)))
        intxt(nonchar(i)) = ' ';
    end
end
outtxt = intxt;

end


%% ---------------------
%% function DOCPTHFORMAT
%% ---------------------
%%
function outtxt = docpthformat(pth, ModelName, lngth)
% add "Library Block" designation to model path for cals/probes in a library block
% add documentation number to front of model path text
outtxt = pth;
path = pth;
if ~ strcmp(ModelName, pth)
    if ~ strcmp(get_param(pth, 'ReferenceBlock'), '')
        parpath = get_param(pth, 'Parent');
        while ~ strcmp(ModelName, parpath) && ~ strcmp(get_param(parpath, 'ReferenceBlock'), '')
            parpath = get_param(parpath, 'Parent');
        end
        outtxt = [path ' (Library Block)'];
        path = parpath;
    end
end
outtxt = strrep(strrep(outtxt, ' / ', '/'), '/', ' / ');

try
    autodocblklst = find_system(path, 'SearchDepth', 1, 'LookUnderMasks', 'all', 'ReferenceBlock', 'AutoDoc_lib/motohawk_autodoc');
    UserData = get_param(autodocblklst{1}, 'UserData');
    docpath = strrep(UserData.DocPath, '.0', '.');
    if ~ strncmp(docpath, '00', 2)
        if strncmp(docpath, '0', 1)
            docpath = docpath(2:end);
        end
        outtxt = [docpath ' ' textend(outtxt, (lngth - length(docpath) - 1), 1)];
    else
        outtxt = textend(outtxt, lngth, 1);
    end
catch
    outtxt = textend(outtxt, lngth, 1);
end
end